import Element from '../element/Element';
import HTMLElement from '../html-element/HTMLElement';
import TextNode from '../text-node/TextNode';
import CommentNode from '../comment-node/CommentNode';
import Window from '../../../window/Window';
import Node from '../node/Node';
import TreeWalker from '../../../tree-walker/TreeWalker';
import DocumentFragment from '../document-fragment/DocumentFragment';
import Event from '../../../event/Event';
import DOMImplementation from '../../../dom-implementation/DOMImplementation';
import INodeFilter from '../../../tree-walker/INodeFilter';
/**
 * Document.
 */
export default class Document extends DocumentFragment {
    defaultView: Window;
    nodeType: number;
    protected _isConnected: boolean;
    protected _isFirstWrite: boolean;
    protected _isFirstWriteAfterOpen: boolean;
    implementation: DOMImplementation;
    /**
     * Creates an instance of Document.
     *
     * @param window Window instance.
     */
    constructor(window: Window);
    /**
     * Node name.
     *
     * @return Node name.
     */
    get nodeName(): string;
    /**
     * Returns <html> element.
     *
     * @return Element.
     */
    get documentElement(): HTMLElement;
    /**
     * Returns <body> element.
     *
     * @return Element.
     */
    get body(): HTMLElement;
    /**
     * Returns <head> element.
     *
     * @return Element.
     */
    get head(): HTMLElement;
    /**
     * Replaces the document HTML with new HTML.
     *
     * @param html HTML.
     */
    write(html: string): void;
    /**
     * Opens the document.
     *
     * @returns Document.
     */
    open(): Document;
    /**
     * Closes the document.
     */
    close(): void;
    /**
     * Creates an element.
     *
     * @param  tagName Tag name.
     * @return Element.
     */
    createElement(tagName: string): Element;
    /**
     * Creates a text node.
     *
     * @param  data Text data.
     * @returns Text node.
     */
    createTextNode(data: string): TextNode;
    /**
     * Creates a comment node.
     *
     * @param  data Text data.
     * @returns Text node.
     */
    createComment(data: string): CommentNode;
    /**
     * Creates a document fragment.
     *
     * @returns Document fragment.
     */
    createDocumentFragment(): DocumentFragment;
    /**
     * Creates a Tree Walker.
     *
     * @param root Root.
     * @param [whatToShow] What to show.
     * @param [filter] Filter.
     */
    createTreeWalker(root: Node, whatToShow?: number, filter?: INodeFilter): TreeWalker;
    /**
     * Creates an event.
     *
     * @legacy
     * @param _type Type.
     * @returns Event.
     */
    createEvent(_type: string): Event;
    /**
     * Imports a node.
     *
     * @param node Node to import.
     * @param Imported node.
     */
    importNode(node: Node): Node;
    /**
     * Returns the element class for a tag name.
     *
     * @param tagName Tag name.
     * @returns Element class.
     */
    private getElementClass;
}
