"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var HTMLElement_1 = __importDefault(require("../html-element/HTMLElement"));
var TextNode_1 = __importDefault(require("../text-node/TextNode"));
var CommentNode_1 = __importDefault(require("../comment-node/CommentNode"));
var Node_1 = __importDefault(require("../node/Node"));
var TreeWalker_1 = __importDefault(require("../../../tree-walker/TreeWalker"));
var DocumentFragment_1 = __importDefault(require("../document-fragment/DocumentFragment"));
var HTMLParser_1 = __importDefault(require("../../../html-parser/HTMLParser"));
var Event_1 = __importDefault(require("../../../event/Event"));
var DOMImplementation_1 = __importDefault(require("../../../dom-implementation/DOMImplementation"));
var HTMLElementTag_1 = __importDefault(require("../../../html-config/HTMLElementTag"));
/**
 * Document.
 */
var Document = /** @class */ (function (_super) {
    __extends(Document, _super);
    /**
     * Creates an instance of Document.
     *
     * @param window Window instance.
     */
    function Document(window) {
        var _this = _super.call(this) || this;
        _this.nodeType = Node_1.default.DOCUMENT_NODE;
        _this._isConnected = true;
        _this._isFirstWrite = true;
        _this._isFirstWriteAfterOpen = false;
        _this.defaultView = window;
        _this.implementation = new DOMImplementation_1.default(window);
        var documentElement = _this.createElement('html');
        var bodyElement = _this.createElement('body');
        var headElement = _this.createElement('head');
        _this.appendChild(documentElement);
        documentElement.appendChild(headElement);
        documentElement.appendChild(bodyElement);
        return _this;
    }
    Object.defineProperty(Document.prototype, "nodeName", {
        /**
         * Node name.
         *
         * @return Node name.
         */
        get: function () {
            return '#document';
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Document.prototype, "documentElement", {
        /**
         * Returns <html> element.
         *
         * @return Element.
         */
        get: function () {
            return this.querySelector('html');
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Document.prototype, "body", {
        /**
         * Returns <body> element.
         *
         * @return Element.
         */
        get: function () {
            return this.querySelector('body');
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Document.prototype, "head", {
        /**
         * Returns <head> element.
         *
         * @return Element.
         */
        get: function () {
            return this.querySelector('head');
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Replaces the document HTML with new HTML.
     *
     * @param html HTML.
     */
    Document.prototype.write = function (html) {
        var root = HTMLParser_1.default.parse(this, html);
        if (this._isFirstWrite || this._isFirstWriteAfterOpen) {
            if (this._isFirstWrite) {
                if (!this._isFirstWriteAfterOpen) {
                    this.open();
                }
                this._isFirstWrite = false;
            }
            this._isFirstWriteAfterOpen = false;
            var documentElement = root.querySelector('html');
            if (documentElement) {
                this.appendChild(documentElement);
            }
            else {
                var documentElement_1 = this.createElement('html');
                var bodyElement = this.createElement('body');
                var headElement = this.createElement('head');
                for (var _i = 0, _a = root.childNodes.slice(); _i < _a.length; _i++) {
                    var child = _a[_i];
                    bodyElement.appendChild(child);
                }
                documentElement_1.appendChild(headElement);
                documentElement_1.appendChild(bodyElement);
                this.appendChild(documentElement_1);
            }
        }
        else {
            var bodyNode = root.querySelector('body');
            for (var _b = 0, _c = (bodyNode || root).childNodes.slice(); _b < _c.length; _b++) {
                var child = _c[_b];
                this.body.appendChild(child);
            }
        }
    };
    /**
     * Opens the document.
     *
     * @returns Document.
     */
    Document.prototype.open = function () {
        this._isFirstWriteAfterOpen = true;
        for (var _i = 0, _a = Object.keys(this._listeners); _i < _a.length; _i++) {
            var eventType = _a[_i];
            var callbacks = this._listeners[eventType];
            if (callbacks) {
                for (var _b = 0, callbacks_1 = callbacks; _b < callbacks_1.length; _b++) {
                    var callback = callbacks_1[_b];
                    this.removeEventListener(eventType, callback);
                }
            }
        }
        for (var _c = 0, _d = this.childNodes.slice(); _c < _d.length; _c++) {
            var child = _d[_c];
            this.removeChild(child);
        }
        return this;
    };
    /**
     * Closes the document.
     */
    Document.prototype.close = function () { };
    /**
     * Creates an element.
     *
     * @param  tagName Tag name.
     * @return Element.
     */
    Document.prototype.createElement = function (tagName) {
        var customElementClass = this.defaultView.customElements.get(tagName);
        var elementClass = customElementClass ? customElementClass : this.getElementClass(tagName);
        elementClass.ownerDocument = this;
        var element = new elementClass();
        element.tagName = tagName.toUpperCase();
        element.ownerDocument = this;
        return element;
    };
    /**
     * Creates a text node.
     *
     * @param  data Text data.
     * @returns Text node.
     */
    Document.prototype.createTextNode = function (data) {
        TextNode_1.default.ownerDocument = this;
        var textNode = new TextNode_1.default();
        textNode.textContent = data;
        return textNode;
    };
    /**
     * Creates a comment node.
     *
     * @param  data Text data.
     * @returns Text node.
     */
    Document.prototype.createComment = function (data) {
        CommentNode_1.default.ownerDocument = this;
        var commentNode = new CommentNode_1.default();
        commentNode.textContent = data;
        return commentNode;
    };
    /**
     * Creates a document fragment.
     *
     * @returns Document fragment.
     */
    Document.prototype.createDocumentFragment = function () {
        DocumentFragment_1.default.ownerDocument = this;
        var documentFragment = new DocumentFragment_1.default();
        return documentFragment;
    };
    /**
     * Creates a Tree Walker.
     *
     * @param root Root.
     * @param [whatToShow] What to show.
     * @param [filter] Filter.
     */
    Document.prototype.createTreeWalker = function (root, whatToShow, filter) {
        if (whatToShow === void 0) { whatToShow = -1; }
        if (filter === void 0) { filter = null; }
        return new TreeWalker_1.default(root, whatToShow, filter);
    };
    /**
     * Creates an event.
     *
     * @legacy
     * @param _type Type.
     * @returns Event.
     */
    Document.prototype.createEvent = function (_type) {
        return new Event_1.default('init');
    };
    /**
     * Imports a node.
     *
     * @param node Node to import.
     * @param Imported node.
     */
    Document.prototype.importNode = function (node) {
        if (!(node instanceof Node_1.default)) {
            throw new Error('Parameter 1 was not of type Node.');
        }
        var clone = node.cloneNode(true);
        clone.ownerDocument = this;
        return clone;
    };
    /**
     * Returns the element class for a tag name.
     *
     * @param tagName Tag name.
     * @returns Element class.
     */
    Document.prototype.getElementClass = function (tagName) {
        return HTMLElementTag_1.default[tagName] || HTMLElement_1.default;
    };
    return Document;
}(DocumentFragment_1.default));
exports.default = Document;
//# sourceMappingURL=Document.js.map