import HTMLElement from '../../basic/html-element/HTMLElement';
import HTMLFormElement from '../form/HTMLFormElement';
import ValidityState from './ValidityState';
/**
 * HTMLElement.
 */
export default class HTMLInputElement extends HTMLElement {
    form: HTMLFormElement;
    formAction: string;
    formMethod: string;
    formNoValidate: boolean;
    name: string;
    type: string;
    disabled: boolean;
    autofocus: boolean;
    required: boolean;
    _value: string;
    _checked: boolean;
    defaultChecked: any;
    indeterminate: boolean;
    alt: string;
    height: number;
    src: string;
    width: number;
    accept: string;
    allowdirs: string;
    autocomplete: string;
    min: string;
    max: string;
    minLength: number;
    maxLength: number;
    pattern: string;
    placeholder: string;
    readOnly: boolean;
    size: number;
    selectionStart: number;
    selectionEnd: number;
    selectionDirection: string;
    defaultValue: string;
    multiple: boolean;
    files: object[];
    step: string;
    inputmode: string;
    /**
     * Returns value.
     *
     * @return Value.
     */
    get value(): string;
    /**
     * Sets value.
     *
     * @param value Value.
     */
    set value(value: string);
    /**
     * Returns checked.
     *
     * @return Checked.
     */
    get checked(): boolean;
    /**
     * Sets checked.
     *
     * @param checked Checked.
     */
    set checked(checked: boolean);
    /**
     * Returns validity state.
     *
     * @returns Validity state.
     */
    get validity(): ValidityState;
    /**
     * Returns validity message.
     *
     * @return Validation message.
     */
    get validationMessage(): string;
    /**
     * Returns "true" if it will validate.
     *
     * @return "true" if it will validate.
     */
    get willValidate(): boolean;
    /**
     * Returns value as Date.
     *
     * @returns Date.
     */
    get valueAsDate(): Date;
    /**
     * Returns value as number.
     *
     * @return Number.
     */
    get valueAsNumber(): number;
    /**
     * Checks validity.
     *
     * @return "true" if validation does'nt fail.
     */
    checkValidity(): boolean;
    /**
     * @override
     */
    _setAttributeProperty(name: string, value: string): void;
}
