import Node from '../../basic/node/Node';
import HTMLElement from '../../basic/html-element/HTMLElement';
import DocumentFragment from '../../basic/document-fragment/DocumentFragment';
/**
 * HTMLTemplateElement.
 */
export default class HTMLTemplateElement extends HTMLElement {
    private contentElement;
    /**
     * Returns the content.
     *
     * @returns Content.
     */
    get content(): DocumentFragment;
    /**
     * Previous sibling.
     *
     * @return Node.
     */
    get previousSibling(): Node;
    /**
     * Next sibling.
     *
     * @return Node.
     */
    get nextSibling(): Node;
    /**
     * First child.
     *
     * @return Node.
     */
    get firstChild(): Node;
    /**
     * Last child.
     *
     * @return Node.
     */
    get lastChild(): Node;
    /**
     * Append a child node to childNodes.
     *
     * @param  node Node to append.
     * @return Appended node.
     */
    appendChild(node: Node): Node;
    /**
     * Remove Child element from childNodes array.
     *
     * @param node Node to remove
     */
    removeChild(node: Node): void;
    /**
     * Inserts a node before another.
     *
     * @param newNode Node to insert.
     * @param referenceNode Node to insert before.
     * @return Inserted node.
     */
    insertBefore(newNode: Node, referenceNode: Node): Node;
    /**
     * Replaces a node with another.
     *
     * @param newChild New child.
     * @param oldChild Old child.
     * @return Replaced node.
     */
    replaceChild(newChild: Node, oldChild: Node): Node;
}
