/// <reference types="node" />
import CustomElementRegistry from '../custom-element/CustomElementRegistry';
import Document from '../nodes/basic/document/Document';
import Node from '../nodes/basic/node/Node';
import TextNode from '../nodes/basic/text-node/TextNode';
import CommentNode from '../nodes/basic/comment-node/CommentNode';
import ShadowRoot from '../nodes/basic/shadow-root/ShadowRoot';
import Element from '../nodes/basic/element/Element';
import DocumentFragment from '../nodes/basic/document-fragment/DocumentFragment';
import TreeWalker from '../tree-walker/TreeWalker';
import Event from '../event/Event';
import CustomEvent from '../event/CustomEvent';
import EventTarget from '../event/EventTarget';
import URL from '../location/URL';
import Location from '../location/Location';
import MutationObserver from '../mutation-observer/MutationObserver';
import DOMParser from '../dom-parser/DOMParser';
/**
 * Handles the Window.
 */
export default class Window extends EventTarget implements NodeJS.Global {
    Node: typeof Node;
    TextNode: typeof TextNode;
    CommentNode: typeof CommentNode;
    ShadowRoot: typeof ShadowRoot;
    Element: typeof Element;
    DocumentFragment: typeof DocumentFragment;
    NodeFilter: {
        FILTER_ACCEPT: number;
        FILTER_REJECT: number;
        FILTER_SKIP: number;
        SHOW_ALL: number;
        SHOW_ATTRIBUTE: number;
        SHOW_CDATA_SECTION: number;
        SHOW_COMMENT: number;
        SHOW_DOCUMENT: number;
        SHOW_DOCUMENT_FRAGMENT: number;
        SHOW_DOCUMENT_TYPE: number;
        SHOW_ELEMENT: number;
        SHOW_ENTITY: number;
        SHOW_ENTITY_REFERENCE: number;
        SHOW_NOTATION: number;
        SHOW_PROCESSING_INSTRUCTION: number;
        SHOW_TEXT: number;
    };
    TreeWalker: typeof TreeWalker;
    DOMParser: typeof DOMParser;
    MutationObserver: typeof MutationObserver;
    Document: typeof Document;
    Event: typeof Event;
    CustomEvent: typeof CustomEvent;
    URL: typeof URL;
    Location: typeof Location;
    CustomElementRegistry: typeof CustomElementRegistry;
    Window: typeof Window;
    Headers: MapConstructor;
    document: Document;
    customElements: CustomElementRegistry;
    location: Location;
    navigator: {
        userAgent: string;
    };
    console: Console;
    self: Window;
    top: Window;
    window: Window;
    Array: ArrayConstructor;
    ArrayBuffer: ArrayBufferConstructor;
    Boolean: BooleanConstructor;
    Buffer: any;
    DataView: DataViewConstructor;
    Date: DateConstructor;
    Error: ErrorConstructor;
    EvalError: EvalErrorConstructor;
    Float32Array: Float32ArrayConstructor;
    Float64Array: Float64ArrayConstructor;
    Function: FunctionConstructor;
    GLOBAL: any;
    Infinity: number;
    Int16Array: Int16ArrayConstructor;
    Int32Array: Int32ArrayConstructor;
    Int8Array: Int8ArrayConstructor;
    Intl: typeof Intl;
    JSON: JSON;
    Map: MapConstructor;
    Math: Math;
    NaN: number;
    Object: ObjectConstructor;
    Number: NumberConstructor;
    Promise: PromiseConstructor;
    RangeError: RangeErrorConstructor;
    ReferenceError: ReferenceErrorConstructor;
    RegExp: RegExpConstructor;
    Set: SetConstructor;
    Symbol: Function & SymbolConstructor;
    SyntaxError: SyntaxErrorConstructor;
    String: StringConstructor;
    TypeError: TypeErrorConstructor;
    URIError: URIErrorConstructor;
    Uint16Array: Uint16ArrayConstructor;
    Uint32Array: Uint32ArrayConstructor;
    Uint8Array: Uint8ArrayConstructor;
    Uint8ClampedArray: Uint8ClampedArrayConstructor;
    WeakMap: WeakMapConstructor;
    WeakSet: WeakSetConstructor;
    clearImmediate: any;
    decodeURI: typeof decodeURI;
    decodeURIComponent: typeof decodeURIComponent;
    encodeURI: typeof encodeURI;
    encodeURIComponent: typeof encodeURIComponent;
    escape: ((str: string) => string) & typeof escape;
    eval: typeof eval;
    global: any;
    isFinite: typeof isFinite;
    isNaN: typeof isNaN;
    parseFloat: typeof parseFloat;
    parseInt: typeof parseInt;
    process: any;
    root: any;
    setImmediate: any;
    queueMicrotask: typeof queueMicrotask;
    undefined: any;
    unescape: ((str: string) => string) & typeof unescape;
    gc: any;
    v8debug: any;
    /**
     * Constructor.
     */
    constructor();
    /**
     * Returns an object containing the values of all CSS properties of an element.
     *
     * @note This method has not been implemented. It is just here for compatibility.
     * @returns Empty object.
     */
    getComputedStyle(): {};
    /**
     * Scrolls to a particular set of coordinates in the document.
     *
     * @note This method has not been implemented. It is just here for compatibility.
     */
    scrollTo(): void;
    /**
     * Disposes the window.
     */
    dispose(): void;
    /**
     * Sets a timer which executes a function once the timer expires.
     *
     * @param callback Function to be executed.
     * @param [delay] Delay in ms.
     * @return Timeout ID.
     */
    setTimeout(callback: () => void, delay?: number): NodeJS.Timeout;
    /**
     * Cancels a timeout previously established by calling setTimeout().
     *
     * @param id ID of the timeout.
     */
    clearTimeout(id: NodeJS.Timeout): void;
    /**
     * Calls a function with a fixed time delay between each call.
     *
     * @param callback Function to be executed.
     * @param [delay] Delay in ms.
     * @return Interval ID.
     */
    setInterval(callback: () => void, delay?: number): NodeJS.Timeout;
    /**
     * Cancels a timed repeating action which was previously established by a call to setInterval().
     *
     * @param id ID of the interval.
     */
    clearInterval(id: NodeJS.Timeout): void;
    /**
     * Mock animation frames with timeouts.
     *
     * @param {function} callback Callback.
     * @returns {NodeJS.Timeout} Timeout ID.
     */
    requestAnimationFrame(callback: (timestamp: number) => void): NodeJS.Timeout;
    /**
     * Mock animation frames with timeouts.
     *
     * @param {NodeJS.Timeout} id Timeout ID.
     */
    cancelAnimationFrame(id: any): void;
    /**
     * Fetch is not supported by the synchronous "Window". Use "AsyncWindow" instead to get support for fetch.
     *
     * @throws Error.
     * @param _url URL to resource.
     * @param [_options] Options.
     * @returns Promise.
     */
    fetch(_url: string, _options: object): Promise<void>;
}
